module RbEvent
	VERSION = '0.3.2'.freeze
	
	# Adds an event listener
	# 
	# listener 'or' listener_block required.
	# 
	# @param [#to_sym] name the name of event
	# @param [#call] listener
	# @return [#call] The listener added
	# 
	# @raise [ArgumentError]
	def listen(name, listener=nil, &listener_block)
		if listener.nil? && listener_block.nil?
			raise ArgumentError, "listener is nil and block is not given"
		elsif !listener.nil? && !listener_block.nil?
			raise ArgumentError, "Both listener and block is given"
		end
		
		if !listener.nil? && !listener.respond_to?(:call)
			raise ArgumentError, "Invalid listener: #{listener.inspect} not responded to :call"
		end
		listener ||= listener_block
		
		name = name.to_sym
		unless rb_event_listeners.key?(name)
			rb_event_listeners[name] = []
		end
		rb_event_listeners[name] << listener
		listener
	end
	alias_method :add_listener, :listen
	
	# Removes an event listener
	# 
	# @param [#to_sym] name The name of event
	# @param [#call] listener
	# @return [void]
	def unlisten(name, listener)
		name = name.to_sym
		return nil unless rb_event_listeners.key?(name)
		rb_event_listeners[name].delete listener
		if rb_event_listeners[name].empty?
			rb_event_listeners.delete(name)
		end
		nil
	end
	alias_method :remove_listener, :unlisten
	
	# Check the listener is added
	# 
	# @param [#to_sym] name The name of event
	# @param [#call] listener
	# @return [Boolean]
	def listen?(name, listener)
		name = name.to_sym
		return false unless rb_event_listeners.key?(name)
		rb_event_listeners[name].include? listener
	end
	alias_method :has_listener?, :listen?

private
	# Gets event listeners
	# @return [Hash]
	# @note rb_event_listeners[<event-name>]
	def rb_event_listeners
		@rb_event_listeners ||= Hash.new
	end
	
protected
	# Calls listeners
	# @param [#to_sym] name
	# @param [Array] args
	def fire_event(name, *args)
		name = name.to_sym
		return nil unless rb_event_listeners.key?(name)
		rb_event_listeners[name].each { |e| e.call(*args) }
		nil
	end
end
